
#include <iostream>
#include <fstream>
#include <TApplication.h>
#include <TROOT.h>
#include <TCanvas.h>
#include <TH1F.h>

#include "ComponentNeBem3dMap.hh"
#include "ViewField.hh"
#include "MediumMagboltz.hh"
#include "MediumSilicon.hh"
#include "Sensor.hh"
#include "AvalancheMicroscopic.hh"
#include "AvalancheMC.hh"
#include "Random.hh"
#include "Plotting.hh"

using namespace Garfield;

int main(int argc, char * argv[]) {

  TApplication app("app", &argc, argv);
  plottingEngine.SetDefaultStyle();

  const bool debug = true;
  
// Can these be made available from the VoxelField file?
// That will considerably reduce the chances of making mistakes.
/*
  const unsigned int nX = 29;
  const unsigned int nY = 29;
  const unsigned int nZ = 201;
  const double xMin = -70.0e-4; 
  const double xMax = 70.0e-4;
  const double yMin = -70.0e-4;
  const double yMax = 70.0e-4;
  const double zMin = 0.0e-4; 
  const double zMax = 1060.0e-4;

  ComponentVoxel* eField = new ComponentVoxel();
  eField->SetMesh(nX, nY, nZ, xMin, xMax, yMin, yMax, zMin, zMax);
  // eField->LoadElectricField("GEMVoxel_EField_Position.txt", "XYZ", true,true,
		 		// 1.0, 1.0, 1.0);
  eField->LoadElectricField("VoxelFPRFine.out", "XYZ", true, true,
		 		1.0, 1.0, 1.0);
*/
  std::string version;
  int optmap, optstgrmap;
  unsigned int nX, nY, nZ;
  double xMin, xMax, yMin, yMax, zMin, zMax;
  double xStgr, yStgr, zStgr;
  std::string mapfile;

  ComponentNeBem3dMap* eField = new ComponentNeBem3dMap();
  eField->LoadMapInfo("MapInfo.out", version, optmap, optstgrmap, nX, nY, nZ,
												xMin, xMax, yMin, yMax, zMin, zMax,
                        xStgr, yStgr, zStgr, mapfile);
  std::cout << "version: " << version << std::endl;
  std::cout << "optmap: " << optmap << std::endl;
  std::cout << "optstgrmap: " << optstgrmap << std::endl;
  std::cout << "xMin xMax: " << xMin << " " << xMax << std::endl;
  std::cout << "yMin yMax: " << yMin << " " << yMax << std::endl;
  std::cout << "zMin zMax: " << zMin << " " << zMax << std::endl;
  std::cout << "xStgr: " << xStgr << std::endl;
  std::cout << "yStgr: " << yStgr << std::endl;
  std::cout << "zStgr: " << zStgr << std::endl;
  std::cout << "nX nY nZ: " << nX << " " << nY << " " << nZ << std::endl;
  std::cout << "mapfile: " << mapfile << std::endl;

  eField->SetMesh(nX, nY, nZ, xMin, xMax, yMin, yMax, zMin, zMax);
  eField->LoadElectricField(mapfile, "XYZ", true, true, 1.0, 1.0, 1.0);
  
  eField->EnablePeriodicityX(); // simple periodicity in X and Y
  eField->EnablePeriodicityY();
  eField->PrintRegions(); 

  // Dimensions of the GEM [cm]
  const double pitch = 140.0e-4;
  const double kapton = 50.e-4;
  const double metal = 5.e-4;

  // setup silicon medium
  MediumSilicon *si = new MediumSilicon();
  si->SetTemperature(253);

 // Setup the gas.
  MediumMagboltz* gas = new MediumMagboltz();
  gas->SetComposition("ar", 80., "co2", 20.);
  gas->SetTemperature(293.15);
  gas->SetPressure(760.);
	// gas->DisableEnergyRangeAdjustment();
  gas->EnableDebugging();
  gas->Initialise();  
  gas->DisableDebugging();
  // Set the Penning transfer efficiency.
  const double rPenning = 0.51;
  const double lambdaPenning = 0.;
  gas->EnablePenningTransfer(rPenning, lambdaPenning, "ar");
  // Load the ion mobilities.
  const std::string path = getenv("GARFIELD_HOME");
  gas->LoadIonMobility(path + "/Data/IonMobility_Ar+_Ar.txt");
  // Associate the gas with the corresponding field map material. 
  // const unsigned int nMaterials = eField->GetNumberOfMaterials();
  // for (unsigned int i = 0; i < nMaterials; ++i) {
    // const double eps = eField->GetPermittivity(i);
    // if (eps == 1.) eField->SetMedium(i, gas);
  // }
  // eField->PrintMaterials();
  eField->SetMedium(0, gas);
  // eField->SetMedium(1, si);
  // eField->SetMedium(2, si);
  std::cout << "Gas has been created and set as medium zero for eField \n";

  // Create the sensor.
  Sensor* sensor = new Sensor();
  sensor->AddComponent(eField);
  sensor->SetArea(-5 * pitch, -5 * pitch, 0.0,
                   5 * pitch,  5 * pitch,  0.1);
  std::cout << "Sensor has been created \n" ;

  // plot the field and sensor
  const bool plotField = true;
  if (plotField) {
    ViewField* fieldView = new ViewField();
    fieldView->SetComponent(eField);
    fieldView->SetPlane(0., -1., 0., 0., 0., 0.);
    fieldView->SetArea(-pitch / 2., 0.0e-4, pitch / 2., 1000.0e-4);
    fieldView->SetVoltageRange(-250., 1000.);
  fieldView->SetElectricFieldRange(0.0, 100.0e3);
    TCanvas* cf = new TCanvas();
    fieldView->SetCanvas(cf);
    fieldView->PlotContour();
  fieldView->SetSensor(sensor);
  fieldView->PlotContour("e");
	cf->SaveAs("Field.pdf");
  }

  AvalancheMicroscopic* aval = new AvalancheMicroscopic();
  // aval->EnableDebugging();
  aval->SetSensor(sensor);
  aval->EnableAvalancheSizeLimit(10000);

  AvalancheMC* drift = new AvalancheMC();
  drift->SetSensor(sensor);
  drift->SetDistanceSteps(2.e-4);

  const bool plotDrift = true;	// to be plotted at the end of the events
  ViewDrift* driftView = new ViewDrift();
  if (plotDrift) {
    driftView->SetArea(-2 * pitch, -2 * pitch, 0.0,
                        2 * pitch,  2 * pitch,  0.1); 
    aval->EnablePlotting(driftView);
    drift->EnablePlotting(driftView);
  }

  const int nEvents = 10;
  for (int i = nEvents; i--;) { 
    std::cout << "\nEVENT: " << i<<"\n";
    if (debug || i % 10 == 0) std::cout << i << "/" << nEvents << "\n";
    // Randomize the initial position. 
    double x0 = -pitch / 2. + RndmUniform() * pitch;
    double y0 = -pitch / 2. + RndmUniform() * pitch;
    // x0 = 0.0; y0 = 0.0;
    double z0 = 750.0e-4; 
    double t0 = 0.;
    double e0 = 0.1;
    aval->AvalancheElectron(x0, y0, z0, t0, e0, 0., 0., 0.);
    int ne = 0, ni = 0;
    aval->GetAvalancheSize(ne, ni);
    const int np = aval->GetNumberOfElectronEndpoints();
    double xe1, ye1, ze1, te1, e1;
    double xe2, ye2, ze2, te2, e2;
    double xi1, yi1, zi1, ti1;
    double xi2, yi2, zi2, ti2;
    int status;
    for (int j = np; j--;) {
      aval->GetElectronEndpoint(j, xe1, ye1, ze1, te1, e1, 
                                   xe2, ye2, ze2, te2, e2, status);
     // std::cout << "\n\n j=" <<j;
      //std::cout << "\n xe1=" <<xe1;
      //std::cout << "\n ye1=" <<ye1;      
      //std::cout << "\n ze1=" <<ze1;
      //std::cout << "\n te1=" <<te1;
      //std::cout << "\n e1=" <<e1;
      //std::cout << "\n\n xe2=" <<xe2;
      //std::cout << "\n ye2=" <<ye2;
      //std::cout << "\n ze2=" <<ze2; 
      //std::cout << "\n te2=" <<te2;     
      //std::cout << "\n e2=" <<e2;
      //std::cout << "\n\n status=" <<status;

      drift->DriftIon(xe1, ye1, ze1, te1);
      drift->GetIonEndpoint(0, xi1, yi1, zi1, ti1, 
                               xi2, yi2, zi2, ti2, status);
      
      //std::cout << "\n xi1=" <<xi1;
      //std::cout << "\n yi1=" <<yi1;      
      //std::cout << "\n zi1=" <<zi1;
      //std::cout << "\n ti1=" <<ti1;
      //std::cout << "\n\n xi2=" <<xi2;
      //std::cout << "\n yi2=" <<yi2;
      //std::cout << "\n zi2=" <<zi2; 
      //std::cout << "\n ti2=" <<ti2;     
      //std::cout << "\n\n status=" <<status;
      //std::cout << "\n****************************************";

    }
  }
  
  if (plotDrift) {
    TCanvas* cd = new TCanvas();
    driftView->SetCanvas(cd);
    driftView->Plot(0);
		cd->SaveAs("Drift.pdf");
  }

  app.Run(kTRUE);

}
